import { pipe } from "@reactive-js/pipe";
import { AbstractDelegatingSubscriber, createSubject, publish, toSafeSubscriber, } from "@reactive-js/observable";
export class AsyncEnumeratorImpl extends AbstractDelegatingSubscriber {
    constructor(delegate, observable) {
        super(delegate);
        this.observable = observable;
        this.isSynchronous = false;
        this.add(observable);
    }
    get subscriberCount() {
        return this.observable.subscriberCount;
    }
    notify(req) {
        this.delegate.notify(req);
    }
    dispatch(req) {
        this.delegate.dispatch(req);
    }
    enumerate() {
        return this.observable.enumerate();
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
export const createAsyncEnumerator = (operator, scheduler, replayCount = 0) => {
    const subject = createSubject(scheduler);
    const safeSubscriber = toSafeSubscriber(subject);
    const observable = pipe(subject, operator, publish(scheduler, replayCount));
    return new AsyncEnumeratorImpl(safeSubscriber, observable);
};
//# sourceMappingURL=createAsyncEnumerator.js.map