import { pipe } from "@reactive-js/pipe";
import { catchError, empty, fromIterator, map, } from "@reactive-js/observable";
import { createAsyncEnumerator } from "./createAsyncEnumerator";
const doneError = Symbol("IteratorDone");
const fromIterableAsyncEnumerator = (iterable, scheduler, replayCount) => {
    const iterator = iterable[Symbol.iterator]();
    const enumerator = fromIterator(iterator).enumerate();
    const operator = (obs) => pipe(obs, map(_ => {
        if (enumerator.move()) {
            return enumerator.current;
        }
        else {
            throw doneError;
        }
    }), catchError(error => (error === doneError ? empty() : undefined)));
    return createAsyncEnumerator(operator, scheduler, replayCount);
};
class FromIterableAsyncEnumerable {
    constructor(iterable) {
        this.iterable = iterable;
    }
    enumerateAsync(scheduler, replayCount) {
        return fromIterableAsyncEnumerator(this.iterable, scheduler, replayCount);
    }
}
export const fromIterable = (iterable) => new FromIterableAsyncEnumerable(iterable);
//# sourceMappingURL=fromIterable.js.map