import { pipe } from "@reactive-js/pipe";
import { publish, } from "@reactive-js/observable";
import { disposableMixin } from "@reactive-js/disposable";
class LiftedAsyncEnumeratorImpl {
    constructor(notify, dispatch, observable, disposable, scheduler) {
        this.notify = notify;
        this.dispatch = dispatch;
        this.observable = observable;
        this.disposable = disposable;
        this.scheduler = scheduler;
        this.add = disposableMixin.add;
        this.dispose = disposableMixin.dispose;
        this.isSynchronous = false;
    }
    get isDisposed() {
        return this.disposable.isDisposed;
    }
    get now() {
        return 0;
    }
    get subscriberCount() {
        return this.observable.subscriberCount;
    }
    enumerate() {
        return this.observable.enumerate();
    }
    schedule(continuation) {
        return this.scheduler.schedule(continuation);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
class LiftedAsyncEnumerable {
    constructor(source, observableOperators, reqOperators) {
        this.source = source;
        this.observableOperators = observableOperators;
        this.reqOperators = reqOperators;
    }
    enumerateAsync(scheduler, replayCount) {
        const enumerator = this.source.enumerateAsync(scheduler);
        const notify = enumerator instanceof LiftedAsyncEnumeratorImpl
            ? enumerator.notify
            : (req) => enumerator.notify(req);
        const liftedNotify = this.reqOperators.reduce((acc, next) => next(acc), notify);
        const dispatch = enumerator instanceof LiftedAsyncEnumeratorImpl
            ? enumerator.dispatch
            : (req) => enumerator.dispatch(req);
        const lifteddispatch = this.reqOperators.reduce((acc, next) => next(acc), dispatch);
        const observable = enumerator.observable || enumerator;
        const liftedObservable = pipe(this.observableOperators.reduce((acc, next) => next(acc), observable), publish(scheduler, replayCount));
        const disposable = enumerator.disposable || enumerator;
        disposable.add(liftedObservable);
        return new LiftedAsyncEnumeratorImpl(liftedNotify, lifteddispatch, liftedObservable, disposable, scheduler);
    }
}
export function lift(op) {
    return iterable => {
        const source = iterable.source || iterable;
        const observableOperators = iterable instanceof LiftedAsyncEnumerable
            ? [...iterable.observableOperators, op]
            : [op];
        const reqOperators = iterable instanceof LiftedAsyncEnumerable ? iterable.reqOperators : [];
        return new LiftedAsyncEnumerable(source, observableOperators, reqOperators);
    };
}
export function liftReq(op) {
    return iterable => {
        const source = iterable.source || iterable;
        const observableOperators = iterable instanceof LiftedAsyncEnumerable
            ? iterable.observableOperators
            : [];
        const reqOperators = iterable instanceof LiftedAsyncEnumerable
            ? [...iterable.reqOperators, op]
            : [op];
        return new LiftedAsyncEnumerable(source, observableOperators, reqOperators);
    };
}
//# sourceMappingURL=lift.js.map