import { map as mapObs, merge, observableMixin, ofValue, onNotify, takeLast, withLatestFrom, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
import { identity } from "./identity";
import { lift } from "./lift";
import { map } from "./map";
class ReduceObservable {
    constructor(enumerable, withLatestSelector, initial) {
        this.enumerable = enumerable;
        this.withLatestSelector = withLatestSelector;
        this.initial = initial;
        this.enumerate = observableMixin.enumerate;
        this.isSynchronous = false;
    }
    subscribe(subscriber) {
        const enumerator = this.enumerable.enumerateAsync(subscriber);
        const eventEmitter = pipe(identity(), lift(onNotify(continueRequest => {
            enumerator.dispatch(continueRequest.req);
        })), map(({ acc }) => acc)).enumerateAsync(subscriber);
        subscriber.add(enumerator).add(eventEmitter);
        pipe(merge(ofValue(this.initial()), pipe(enumerator, withLatestFrom(eventEmitter, this.withLatestSelector))), onNotify(next => {
            if (next.type === 1) {
                eventEmitter.dispatch(next);
            }
            else {
                enumerator.dispose();
            }
        }), mapObs(({ acc }) => acc), takeLast()).subscribe(subscriber);
    }
}
export const reduce = (reducer, initial) => enumerable => {
    const withLatestSelector = (next, acc) => reducer(acc, next);
    return new ReduceObservable(enumerable, withLatestSelector, initial);
};
//# sourceMappingURL=reduce.js.map